@@ -8,21 +8,24 @@ use POE::Component::SmokeBox::Job;
 use POE::Component::SmokeBox::Result;
 use vars qw($VERSION);
 
-$VERSION = '0.01_10';
+$VERSION = '0.43_01';
 
 sub spawn {
   my $package = shift;
   my %params = @_;
   $params{lc $_} = delete $params{$_} for keys %params;
   my $options = delete $params{'options'};
+  $params{'delay'} = 0 unless exists $params{'delay'};
   my $self = bless \%params, $package;
   $self->{session_id} = POE::Session->create(
 	object_states => [
 	   $self => { 
-		shutdown   => '_shutdown',
-		add_smoker => '_add_smoker',
-		del_smoker => '_del_smoker',
-		submit     => '_submit',
+		shutdown      => '_shutdown',
+		add_smoker    => '_add_smoker',
+		del_smoker    => '_del_smoker',
+		submit        => '_submit',
+		register_ui   => '_reg_ui',
+		unregister_ui => '_unreg_ui',
 	   },
 	   $self => [qw(_start)],
 	],
@@ -40,6 +43,20 @@ sub multiplicity {
   return $_[0]->{multiplicity};
 }
 
+sub delay {
+  if ( defined $_[1] ) {
+    # verify it's an int
+    if ( $_[1] !~ /^\d+$/ ) {
+      return;
+    } else {
+      $_[0]->{delay} = $_[1];
+      return $_[1];
+    }
+  } else {
+    return $_[0]->{delay};
+  }
+}
+
 sub queues {
   return map { $_->{queue} } @{ $_[0]->{queues} };
 }
@@ -97,7 +114,9 @@ sub _add_smoker {
   # If multiplicity start a job queue for each smoker object.
   if ( $self->{multiplicity} or scalar @{ $self->{queues} } == 0 ) {
     my $queue = { };
-    $queue->{queue} = POE::Component::SmokeBox::JobQueue->spawn();
+    $queue->{queue} = POE::Component::SmokeBox::JobQueue->spawn(
+      'delay' => $self->{delay},
+    );
     push @{ $queue->{smokers} }, $smoker;
     push @{ $self->{queues} }, $queue;
     return;
@@ -131,7 +150,7 @@ sub _del_smoker {
 
 sub submit {
   my $self = shift;
-  $poe_kernel->call( $self->{session_id}, '_submit', @_ );
+  $poe_kernel->call( $self->{session_id}, 'submit', @_ );
 }
 
 sub _submit {
@@ -174,7 +193,13 @@ sub _submit {
   return;
 }
 
-1;
+sub _reg_ui {
+}
+
+sub _unreg_ui {
+}
+
+"We've Got a Fuzzbox and We're Gonna Use It";
 
 __END__
 
@@ -208,9 +233,9 @@ POE::Component::SmokeBox - POE enabled CPAN smoke testing with added value.
   
   POE::Session->create(
         package_states => [
-           'main' => [ qw(_start _stop _results _recent) ],
+           'main' => [ qw(_start _stop _results) ],
         ],
-        heap => { perl => $perl, pending => { @ARGV },
+        heap => { perl => $perl, pending => [ @ARGV ] },
   );
   
   $poe_kernel->run();
@@ -235,7 +260,7 @@ POE::Component::SmokeBox - POE enabled CPAN smoke testing with added value.
   }
   
   sub _results {
-    my $result = $_[ARG0];
+    my $results = $_[ARG0];
     print $_, "\n" for map { @{ $_->{log} } } $results->{result}->results();
     undef;
   }
@@ -267,7 +292,8 @@ Creates a new session and returns an object. Takes a number of parameters:
   'options', a hashref of POE session options;
   'multiplicity', set to a true value to enable multiplicity, default is false;
   'smokers', an arrayref of POE::Component::SmokeBox::Smoker objects;
-  
+  'delay', the time in seconds to wait between job runs, default is 0;
+
 =back
 
 =head1 METHODS
@@ -282,6 +308,8 @@ Returns the L<POE::Session> ID of the smokebox component.
 
 Returns true or false depending on whether multiplicity is enabled or not.
 
+NOTE: If you enable multiplicity, you cannot use "delay" as an argument to SmokeBox::Job->new!
+
 =item C<queues>
 
 Returns a list of L<POE::Component::SmokeBox::JobQueue> objects that are currently active in the smokebox.
@@ -294,6 +322,11 @@ Takes one mandatory argument, a L<POE::Component::SmokeBox::Smoker> object to ad
 
 Takes one mandatory argument, a L<POE::Component::SmokeBox::Smoker> object to remove from the smokebox.
 
+=item C<delay>
+
+Sets the delay in seconds between job runs. Useful to "throttle" your smoker :) If called with no arguments, returns
+the current delay. This option will work even if multiplicity is enabled.
+
 =item C<submit>
 
 Submits a job to the smokebox. Takes a number of parameters.
@@ -359,6 +392,7 @@ Each result is a hashref:
   'idle_kill', only present if the job was killed because of excessive idle;
   'excess_kill', only present if the job was killed due to excessive runtime;
   'term_kill', only present if the job was killed due to a poco shutdown event;
+  'cb_kill', only present if the job was killed due to the callback returning false;
 
 =head1 AUTHOR
 
