@@ -4,12 +4,52 @@ use strict;
 use warnings;
 use Carp;
 use Storable;
-use POE qw(Wheel::Run);
-use Digest::MD5 qw(md5_hex);
+use File::Temp qw[tempdir];
+use File::Path;
+use File::Spec;
+use POSIX qw( O_CREAT O_RDWR O_RDONLY );         # for SDBM_File
+use SDBM_File;
+use POE qw[Wheel::Run Filter::Line];
+use Digest::SHA qw[sha256];
+use Env::Sanctify;
 use Module::Pluggable search_path => 'POE::Component::SmokeBox::Backend', sub_name => 'backends', except => 'POE::Component::SmokeBox::Backend::Base';
 use vars qw($VERSION);
 
-$VERSION = '0.01';
+$VERSION = '0.43_01';
+
+my $GOT_KILLFAM;
+my $GOT_PTY;
+
+BEGIN {
+        $GOT_KILLFAM = 0;
+        eval {
+                require Proc::ProcessTable;
+                $GOT_KILLFAM = 1;
+        };
+        $GOT_PTY = 0;
+        eval {
+                require IO::Pty;
+                $GOT_PTY = 1;
+        };
+	if ( $^O eq 'MSWin32' ) {
+		require POE::Wheel::Run::Win32;
+
+		# MSWin32: Disable critical error popups
+		# Thanks to https://rt.cpan.org/Public/Bug/Display.html?id=56547
+
+		# Call kernel32.SetErrorMode(SEM_FAILCRITICALERRORS):
+		# "The system does not display the critical-error-handler message box.
+		# Instead, the system sends the error to the calling process." and
+		# "A child process inherits the error mode of its parent process."
+		if ( eval { require Win32API::File } ) {
+			Win32API::File->import( qw( SetErrorMode SEM_FAILCRITICALERRORS SEM_NOGPFAULTERRORBOX ) );
+			SetErrorMode( SEM_FAILCRITICALERRORS() | SEM_NOGPFAULTERRORBOX() );
+		} else {
+			warn "Unable to use Win32API::File -> $@";
+			warn 'This means sometimes perl.exe will popup a dialog box... Annoying!';
+		}
+	}
+}
 
 my @cmds = qw(check index smoke);
 
@@ -41,10 +81,18 @@ sub spawn {
   }
   $opts{idle} = 600 unless $opts{idle};
   $opts{timeout} = 3600 unless $opts{timeout};
+  $opts{timer} = 60 unless $opts{timer};
   $opts{type} = 'CPANPLUS::YACSmoke' unless $opts{type};
   $opts{command} = lc $opts{command} || 'check';
   $opts{command} = 'check' unless grep { $_ eq $opts{command} } @cmds;
   $opts{perl} = $^X unless $opts{perl}; # and -e $opts{perl};
+  $opts{no_log} = 0 unless $opts{no_log};
+  $opts{check_warnings} = 1 unless exists $opts{check_warnings};
+
+  if ( $opts{check_warnings} ) {
+     require String::Perl::Warnings;
+  }
+
   if ( $opts{command} eq 'smoke' and !$opts{module} ) {
      carp "You must specify a 'module' with 'smoke'\n";
      return;
@@ -122,6 +170,18 @@ sub _start {
   $kernel->refcount_increment( $sender_id, __PACKAGE__ );
   $self->{session} = $sender_id;
   $kernel->detach_myself() if $kernel != $sender;
+
+  $self->{_wheel_log} = [ ];
+
+  {
+    $self->{_tempdir} = File::Temp->newdir();
+    my $file = File::Spec->catfile( $self->{_tempdir}->dirname, 'digests.dat' );
+    $self->{_digests} = { };
+    tie %{ $self->{_digests} }, 'SDBM_File', $file, O_CREAT|O_RDWR, 0644 or die "Could not tie: $!\n";
+  }
+  $self->{_loop_detect} = 0;
+  $self->{start_time} = time();
+
   $kernel->yield( '_spawn_wheel' );
   return;
 }
@@ -135,42 +195,81 @@ sub _shutdown {
 
 sub _spawn_wheel {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
-  # Set appropriate %ENV values before we fork()
-  my %env_back_up;
-  if ( $self->{env} and ref $self->{env} eq 'HASH' ) {
-     # backup the current values;
-     $env_back_up{$_} = delete $ENV{$_} for grep { defined $ENV{$_} } keys %{ $self->{env} };
-     $ENV{$_} = $self->{env}->{$_} for keys %{ $self->{env} };
+
+  # do we need to process callbacks?
+  if ( $self->{do_callback} ) {
+    # Ask it if we should process this job or not?
+    unless ( $self->{do_callback}->( 'BEFORE', $self ) ) {
+      warn "Callback denied job, aborting!\n" if $self->{debug} or $ENV{PERL5_SMOKEBOX_DEBUG};
+      my $job = $self->_finalize_job( -1 );
+      $job->{cb_kill} = 1;
+      $kernel->post( $self->{session}, $self->{event}, $job );
+      return;
+    }
   }
-  $self->{wheel} = POE::Wheel::Run->new(
+
+  # Set appropriate %ENV values before we fork()
+  my $sanctify = Env::Sanctify->sanctify( 
+	env => $self->{env}, 
+	sanctify => [
+			'^POE_',
+			'^PERL5_SMOKEBOX',
+			'^HARNESS_',
+			'^(PERL5LIB|TAP_VERSION|TEST_VERBOSE)$',
+      '^AUTHOR_TESTING$',
+      '^PERL_TEST',
+  ] );
+  my $type = 'POE::Wheel::Run';
+  $type .= '::Win32' if $^O eq 'MSWin32';
+  $self->{wheel} = $type->new(
     Program     => $self->{program},
-#    ProgramArgs => $job->{program_args},
     StdoutEvent => '_wheel_stdout',
     StderrEvent => '_wheel_stderr',
+    StdoutFilter => POE::Filter::Line->new( InputLiteral => "\n" ),
+    StderrFilter => POE::Filter::Line->new( InputLiteral => "\n" ),
     ErrorEvent  => '_wheel_error',
-    CloseEvent  => '_wheel_close',
+    CloseEvent  => '_wheel_closed',
+    ( $GOT_PTY ? ( Conduit => 'pty-pipe' ) : () ),
   );
   # Restore the %ENV values
-  delete $ENV{$_} for keys %{ $self->{env} };
-  $ENV{$_} = $env_back_up{$_} for keys %env_back_up;
-  $self->{_wheel_log} = [ ];
-  $self->{_digests} = { };
-  $self->{_loop_detect} = 0;
-  $self->{start_time} = time();
+  $sanctify->restore();
+  $self->{_wheel_time} = time();
   $self->{PID} = $self->{wheel}->PID();
   $kernel->sig_child( $self->{PID}, '_sig_child' );
-  $kernel->delay( '_wheel_idle', 60 ) unless $self->{command} eq 'index';
+  $kernel->delay( '_wheel_idle', $self->{timer} ) unless $self->{command} eq 'index';
   return;
 }
 
 sub _sig_child {
   my ($kernel,$self,$thing,$pid,$status) = @_[KERNEL,OBJECT,ARG0..ARG2];
-  push @{ $self->{_wheel_log} }, "$thing $pid $status";
+  push @{ $self->{_wheel_log} }, "$thing $pid $status" if ! $self->{no_log};
   warn "$thing $pid $status\n" if $self->{debug} or $ENV{PERL5_SMOKEBOX_DEBUG};
+  $kernel->sig_handled();
   $kernel->delay( '_wheel_idle' );
+
+  my $job = $self->_finalize_job( $status );
+
+  # do we need to process callbacks?
+  if ( $self->{do_callback} ) {
+    # Inform the callback that the job is done
+    $self->{do_callback}->( 'AFTER', $self, $job );
+  }
+
+  $kernel->post( $self->{session}, $self->{event}, $job );
+  return;
+}
+
+sub _finalize_job {
+  my( $self, $status ) = @_;
+
   $self->{end_time} = time();
+  untie %{ $self->{_digests} };
   delete $self->{_digests};
+  my $tempdir = $self->{_tempdir}->dirname();
+  delete $self->{_tempdir};
+  rmtree( $tempdir );
   delete $self->{_loop_detect};
+
   my $job = { };
   $job->{status} = $status;
   $job->{log} = $self->{_wheel_log};
@@ -178,54 +277,74 @@ sub _sig_child {
   $job->{$_} = $self->{$_} for grep { $self->{$_} } qw(command env PID start_time end_time idle_kill excess_kill term_kill perl type);
   $job->{program} = $self->{program} if $self->{debug} or $ENV{PERL5_SMOKEBOX_DEBUG};
   $job->{module} = $self->{module} if $self->{command} eq 'smoke';
-  $kernel->post( $self->{session}, $self->{event}, $job );
-  $kernel->refcount_decrement( $self->{session}, __PACKAGE__ );
-  $kernel->sig_handled();
+  $poe_kernel->refcount_decrement( $self->{session}, __PACKAGE__ );
+
+  return $job;
 }
 
 sub _wheel_error {
-  $poe_kernel->delay( '_wheel_idle' );
-  delete $_[OBJECT]->{wheel};
-  undef;
+  my ($self,$operation,$errnum,$errstr,$wheel_id) = @_[OBJECT,ARG0..ARG3];
+  $errstr = "remote end closed" if $operation eq "read" and !$errnum;
+  warn "wheel $wheel_id generated $operation error $errnum: $errstr\n" if $self->{debug} or $ENV{PERL5_SMOKEBOX_DEBUG};
+  return;
 }
 
 sub _wheel_closed {
-  $poe_kernel->delay( '_wheel_idle' );
-  delete $_[OBJECT]->{wheel};
-  undef;
+  my ($kernel,$self) = @_[KERNEL,OBJECT];
+  $kernel->delay( '_wheel_idle' );
+  warn "wheel closed\n" if $self->{debug} or $ENV{PERL5_SMOKEBOX_DEBUG};
+  delete $self->{wheel};
+  return;
 }
 
 sub _wheel_stdout {
   my ($self, $input, $wheel_id) = @_[OBJECT, ARG0, ARG1];
+  return if $self->{_killed};
   $self->{_wheel_time} = time();
-  push @{ $self->{_wheel_log} }, $input;
+  push @{ $self->{_wheel_log} }, $input if ! $self->{no_log};
   warn $input, "\n" if $self->{debug} or $ENV{PERL5_SMOKEBOX_DEBUG};
-  if ( $self->_detect_loop( $input ) ) {
+  if ( $self->_detect_loop( $input, 'stdout' ) ) {
     $self->{excess_kill} = 1;
     $poe_kernel->yield( '_wheel_kill', 'Killing current run due to detection of looping output' );
   }
-  undef;
+  return;
 }
 
 sub _wheel_stderr {
   my ($self, $input, $wheel_id) = @_[OBJECT, ARG0, ARG1];
+  return if $self->{_killed};
   $self->{_wheel_time} = time();
-  push @{ $self->{_wheel_log} }, $input;
+  push @{ $self->{_wheel_log} }, $input if ! $self->{no_log};
   warn $input, "\n" if $self->{debug} or $ENV{PERL5_SMOKEBOX_DEBUG};
-  if ( $self->_detect_loop( $input ) ) {
+  if ( $self->_detect_loop( $input, 'stderr' ) ) {
     $self->{excess_kill} = 1;
     $poe_kernel->yield( '_wheel_kill', 'Killing current run due to detection of looping output' );
   }
-  undef;
+  return;
 }
 
 sub _detect_loop {
   my $self = shift;
   my $input = shift || return;
+  my $handle = shift || 'stdout';
   return if $self->{_loop_detect};
-  my $digest = md5_hex( $input );
-  $self->{_digests}->{ $digest }++;
-  return unless ++$self->{_digests}->{ $digest } > 300;
+  return if $input =~ /^\[(MSG|ERROR)\]/;
+  my $digest = sha256( $input );
+
+  my $weighting;
+  if ( $self->{check_warnings} ) {
+    $weighting = ( $handle eq 'stderr' and String::Perl::Warnings::is_warning($input) ) ? 1 : 10;
+  } else {
+    $weighting = $handle eq 'stderr' ? 1 : 10;
+  }
+
+  if ( exists $self->{_digests}->{ $digest } ) {
+    $self->{_digests}->{ $digest } += $weighting;
+  }
+  else {
+    $self->{_digests}->{ $digest } = $weighting;
+  }
+  return unless ++$self->{_digests}->{ $digest } > 3000;
   return $self->{_loop_detect} = 1;
 }
 
@@ -248,18 +367,21 @@ sub _wheel_idle {
 
 sub _wheel_kill {
   my ($kernel,$self,$reason) = @_[KERNEL,OBJECT,ARG0];
-  push @{ $self->{_wheel_log} }, $reason;
+  return if $self->{_killed};
+  $self->{_killed} = 1;
+  push @{ $self->{_wheel_log} }, $reason if ! $self->{no_log};
   warn $reason, "\n" if $self->{debug} or $ENV{PERL5_SMOKEBOX_DEBUG};
   if ( $^O eq 'MSWin32' and $self->{wheel} ) {
-#    my $grp_pid = $self->{_current_job}->{GRP_PID};
-#    return unless $grp_pid;
-#    warn Win32::FormatMessage( Win32::GetLastError() )
-#	unless Win32::Process::KillProcess( $grp_pid, 0 );
-    # TODO
+    $self->{wheel}->kill();
   }
   else {
     if ( !$self->{no_grp_kill} ) {
-      $self->{wheel}->kill(-9) if $self->{wheel};
+      if ( $^O eq 'solaris' ) {
+	 kill( 9, '-' . $self->{wheel}->PID() ) if $self->{wheel};
+      }
+      else {
+         $self->{wheel}->kill(-9) if $self->{wheel};
+      }
     }
 #    elsif ( $GOT_KILLFAM ) {
 #      _kill_family( 9, $self->{wheel}->PID() ) if $self->{wheel};
@@ -341,6 +463,7 @@ Creates a new POE::Component::SmokeBox::Backend component. Takes a number of par
   'timeout', change runtime timeout, specified in seconds, default is 3600;
   'module', the module to process, mandatory if 'smoke' command is specified;
   'env', a hashref of %ENV values to set when processing;
+  'no_log', enable to not store the job output log, default is false;
 
 You may also pass in arbitary parameters which will passed back to you in the C<event> specified. These
 arbitary parameters must be prefixed with an underscore.
@@ -401,6 +524,7 @@ ARG0 of the C<event> specified in one of the constructors will be a hashref with
   'idle_kill', only present if the job was killed because of excessive idle;
   'excess_kill', only present if the job was killed due to excessive runtime;
   'term_kill', only present if the job was killed due to a poco shutdown event;
+  'cb_kill', only present if the job was killed due to the callback returning false;
 
 Plus any of the parameters given to one of the constructors, including arbitary ones.
 
