@@ -6,7 +6,7 @@ use Params::Check qw(check);
 use base qw(Object::Accessor);
 use vars qw($VERSION $VERBOSE);
 
-$VERSION = '0.01';
+$VERSION = '0.43_01';
 
 sub new {
   my $package = shift;
@@ -17,13 +17,9 @@ sub new {
 	type    => { defined => 1, default => 'CPANPLUS::YACSmoke', },
 	command => { allow => [ qw(check index smoke) ], default => 'check', },
 	module  => { defined => 1 },
-	smokers => { defined => 1, allow => [
-		sub { 
-			return 1 if ref $_[0] eq 'ARRAY'
-				and scalar @{ $_[0] }
-				and ( grep { $_->isa('POE::Component::SmokeBox::Smoker') } @{ $_[0] } ) == @{ $_[0] };
-		}, 
-	], },
+	no_log	=> { defined => 1, allow => qr/^(?:0|1)$/, default => 0, },
+	delay	=> { defined => 1, allow => qr/^\d+$/, default => 0, },
+	check_warnings => { defined => 1, allow => qr/^(?:0|1)$/, default => 1, },
   };
 
   my $args = check( $tmpl, { @_ }, 1 ) or return;
@@ -38,13 +34,10 @@ sub new {
 	type    => sub { defined $_[0]; },
 	command => [ qw(check index smoke) ],
 	module  => sub { defined $_[0]; },
-	smokers => 
-		sub { 
-			return 1 if ref $_[0] eq 'ARRAY'
-				and scalar @{ $_[0] }
-				and ( grep { $_->isa('POE::Component::SmokeBox::Smoker') } @{ $_[0] } ) == @{ $_[0] };
-		}, 
 	id	=> sub { defined $_[0]; },
+	no_log	=> qr/^(?:0|1)$/,
+	delay	=> qr/^\d+$/,
+	check_warnings => qr/^(?:0|1)$/,
   };
   $self->mk_accessors( $accessor_map );
   $self->$_( $args->{$_} ) for keys %{ $args };
@@ -53,7 +46,7 @@ sub new {
 
 sub dump_data {
   my $self = shift;
-  my @returns = qw(idle timeout type command);
+  my @returns = qw(idle timeout type command no_log check_warnings);
   push @returns, 'module' if $self->command() eq 'smoke';
   return map { ( $_ => $self->$_ ) } @returns;
 }
@@ -92,9 +85,9 @@ Creates a new POE::Component::SmokeBox::Job object. Takes a number of parameters
   'type', the type of backend to use, default is 'CPANPLUS::YACSmoke';
   'command', the command to run, 'check', 'index' or 'smoke', default is 'check';
   'module', the distribution to smoke, mandatory if command is 'smoke';
-  'smokers', an arrayref of POE::Component::SmokeBox::Smoker objects;
-
-C<smokers> is only required if you are directly using L<POE::Component::SmokeBox::JobQueue>
+  'no_log', enable to not store the job output log, default is false;
+  'delay', the time in seconds to wait between smoker runs, default is 0;
+  'check_warnings', enable to check job output for common perl warning strings, default is 1;
 
 =back
 
@@ -124,9 +117,22 @@ The command to run, C<'check'>, C<'index'> or C<'smoke'>, default is C<'check'>.
 
 The distribution to smoke, mandatory if command is C<'smoke'>.
 
-=item C<smokers>
+=item C<no_log>
 
-An arrayref of L<POE::Component::SmokeBox::Smoker> objects
+Boolean value determining whether the job will store it's STDERR/STDOUT log, default 0.
+
+=item C<delay>
+
+Number of seconds to pause between smokers for this job. Useful to "throttle" your smokers! The default is 0.
+
+WARNING: This option is ineffective if you have multiplicity set in SmokeBox.
+
+=item C<check_warnings>
+
+Boolean value determining whether SmokeBox will use L<String::Perl::Warnings> to check job output for common perl warnings.
+
+If enabled, SmokeBox will not kill a job prematurely if it prints a lot of warnings to STDERR. It will run the additional check
+for the warnings and give the process more time to reach the limit. ( look at L<POE::Component::SmokeBox::Backend> for 'excess_kill' )
 
 =item C<dump_data>
 
